/*Treiber fr ein Grafik-LCD mit HD61202*/
/*Matthias Weier (c) 2003*/

#include "mydefs.h"

#if MODULE_GRAFIK == TRUE

#include "func.h"
#include "grafik.h"
#include "debug.h"
#include "rs232.h"
#include "lcd.h"
#include "grafik_font.h"
#include <mcs51reg.h>
#include <string.h>

void grafik_init(void)							//Initialiesiert das Grafikdisplay
{
	
	clvar = clvar & 0xFC;						//RST des Grafikcontrollers
	CLWR = clvar;
	delay_us(5);							//10s Delay

	grafik_wr_cmd(0x3E,0);						//Chip 1 Display aus
	grafik_wr_cmd(0x3E,1);						//Chip 2 Display aus
	delay_us(1000);
	grafik_wr_cmd(0x3F,0);						//Chip 1 Display an
	grafik_wr_cmd(0x3F,1);						//Chip 2 Display an
	grafik_wr_cmd(0xC0,0);						//Chip 1 Startline 0
	grafik_wr_cmd(0xC0,1);						//Chip 2 Startline 0
	grafik_wr_cmd(0xB8,0);						//Chip 1 X-Adresse auf 0
	grafik_wr_cmd(0xB8,1);						//Chip 2 X-Adresse auf 0
	grafik_wr_cmd(0x40,0);						//Chip 1 Y-Adresse auf 0
	grafik_wr_cmd(0x40,1);						//Chip 2 Y-Adresse auf 0

}

/*
	Frgt das Busybit ab. Dabei wird der Zustand des RS-Bit evtl. verndert 
	(Wird auf 0 gesetzt) Sollen danach Daten geschrieben werden mu RS erneut gesetzt werden.
*/
void grafik_wbusy(void)
{
	
	clvar = (clvar | 0x02) & 0xFE;					//Instruktionen fr Grafikcontroller
	CLWR = clvar;
	
	while(((graRD1 & 0x80) == 0x80) || ((graRD2 & 0x80) == 0x80));
	
}

void grafik_clear(void)							//Lscht das gesamte Display	
{
	unsigned char page = 0xB8;
	unsigned char dummy = 0;
	
	for(page = 0xB8; page <= 0xC0; page++)
	{
		grafik_wr_cmd(page,0);
		grafik_wr_cmd(page,1);
		grafik_wr_cmd(0x40,0);					//Y-Adresse Chip 1 = 0
		grafik_wr_cmd(0x40,1);					//Y-Adresse Chip 2 = 0

		for(dummy = 0; dummy <= 120; dummy++)
		{
			grafik_wr_data(0x00,0);
			grafik_wr_data(0x00,1);
		}
	}

}


/*
Gibt ein Byte auf dem Grafikdisplay an definierter Stelle aus.
Parameter:	byte = Das zu schreibende Byte
		page = Bit 0 - 2 Page Bit 4 0 = Chip1 1 = Chip2
		line = 0-63 Linie des Bildes
*/
void grafik_printbyte(unsigned char byte, unsigned char page, unsigned char line)
{

	unsigned char dummy = 0;
	
	dummy = (page & 0x10)>>4;						//Welcher Chip wurde ausgewhlt
	page = page & 0x0F;
	
	grafik_wr_cmd((page | 0xB8),dummy);
	grafik_wr_cmd((line | 0x40),dummy);
	grafik_wr_data(byte,dummy);

}

/*
Liet ein Byte aus dem Grafikdisplay an definierter Stelle aus.
Parameter:	page = Bit 0 - 2 Page Bit 4 0 = Chip1 1 = Chip2
		line = 0-63 Linie des Bildes
*/
unsigned char grafik_readbyte(unsigned char page, unsigned char line)
{

	unsigned char dummy = 0;

	dummy = (page & 0x10)>>4;						//Welcher Chip wurde ausgewhlt
	page = page & 0x0F;
	
	grafik_wr_cmd((page | 0xB8),dummy);
	grafik_wr_cmd((line | 0x40),dummy);
	return grafik_rd_data(dummy);
}

/*
Gibt ein von bmpconv.exe vorbereitetes Bitmap auf dem Grafikdisplay aus. 
Speicherstelle mit dem ersten Byte mu bergeben werden.
*/
void grafik_printbmp(unsigned char *stelle)
{
	unsigned char dummy = 0;
	unsigned char index = 0;
	unsigned char index1 = 0;
	unsigned int addr;

	addr = (unsigned int)stelle;	
	
	grafik_init();
	
	for(index = 0;index <=7; index++)
	{
		grafik_wr_cmd((0xB8 | index),0);				//Page
		
		for (index1 = 0; index1 <= 63; index1++)
		{
			if (grafik_inv == 0) grafik_wr_data(*stelle,0);
			else grafik_wr_data(~(*stelle),0);

			stelle++;
		}
		stelle = stelle + 64;
	}	

	stelle = (unsigned char const code *) addr + 64;
	
	grafik_wr_cmd(0x40,1);							//Y-Adresse

	for(index = 0;index <=7; index++)
	{
		grafik_wr_cmd((0xB8 | index),1);				//Page
		
		for (index1 = 0; index1 <= 63; index1++)
		{
			if (grafik_inv == 0) grafik_wr_data(*stelle,1);
			else grafik_wr_data(~(*stelle),1);

			stelle++;
		}
		stelle = stelle + 64;
	}		
}

/*
Gibt ein Zeichen Pixelweise an der durch x und y adressierten Stelle aus.
*/
void grafik_printcxy(unsigned char x, unsigned char y, unsigned char c)
{
	unsigned char dummy;
	unsigned char i;
	unsigned char temp;

	for(dummy = 0; dummy < 5; dummy++)
	{

		temp = gra_font8x5[c][dummy];
						
		for (i = 0; i <= 7; i++)
		{			
			if ((temp & 0x01) == 0x01) 
			{
				grafik_xy(x,y+i,grafik_inv);
			}
			
			temp = temp>>1;
		}
		
		x++;
	}
}

/*
Gibt eine Zeichenkette ab der durch x und y adressierten Stelle mit Hilfe von
grafik_printcxy aus
*/
void grafik_printtextxy(unsigned char x, unsigned char y, unsigned char *string)
{
	unsigned char index;
	unsigned char count;
	
	count = strlen(string);
	
	for(index = 0; index < count; index++)
	{
		if (x > 0x7B) break;
		
		grafik_printcxy(x,y,*string);
		
		x+=6;
		string++;
	}
}

void grafik_xy(unsigned char x, unsigned char y, bit state)
{
	bit chip2 = 0;							//Bit fr Chiperkennung
	unsigned char page = 0;						//Page im Grafikspeicher
	unsigned char pos;						//Bitposition
	volatile unsigned char dummy = 0;				//Bitposition
	
	if(state == 1) pos = 0xFE;
	else pos = 0x01;
	
	if (x > 63)
	{
		x = x - 64; 
		chip2 = 1;
	}
	
	page = y / 8;
	dummy = y % 8;
	pos = (pos<<dummy)|(pos>>(8-dummy));

	grafik_wr_cmd((0xB8 | page),chip2);
	grafik_wr_cmd((0x40 | x),chip2);
		
	dummy = grafik_rd_data(chip2);					//dummy read
	dummy = grafik_rd_data(chip2);
		
	grafik_wr_cmd((0xB8 | page),chip2);
	grafik_wr_cmd((0x40 | x),chip2);
		
	if(state == 1) dummy = dummy & pos;
	else dummy = dummy | pos;
	
	grafik_wr_data(dummy,chip2);

}

/*
Zeichnet ein Rechteck auf dem Grafik-LCD
relativ langsam da jeder Punkt einzeln gezeichnet wird
Parameter:	x:	Startpunkt in der X-Achse
		y:	Startpunkt in der Y-Achse
		l:	Lnge (Ausdehnung in der X-Achse, positiv)
		h:	Hhe (Ausdehnung in der Y-Achse, positiv)
*/

void grafik_drawrec(unsigned char x, unsigned char y, unsigned char l, unsigned char h)
{
	grafik_line(x,y,x+l,y);
	grafik_line(x,y,x,y+h);
	grafik_line(x+l,y,x+l,y+h);
	grafik_line(x,y+h,x+l,y+h);

}

//Schlagwort Bresenham
//Artur Pundsack
void grafik_line(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
	unsigned char nx;
	unsigned char ny;
	char inc_x;
	char inc_y;
	int tmp1;
	int tmp2;
	int schwelle;

	nx = x1;
	ny = y1;

	tmp1 = x2 - nx;
	tmp2 = y2 - ny;

	inc_x = (tmp1>0)? 1:-1;
	inc_y = (tmp2>0)? 1:-1;

    	tmp2 = (tmp2<0)? (-tmp2):tmp2; 
    	tmp1 = (tmp1<0)? (-tmp1):tmp1;

	if(tmp2 < tmp1 )
	{
		tmp1 = -tmp1;
		tmp2 = tmp2 + tmp2;
		schwelle = tmp1 + tmp1;
		while(nx != x2)
		{
			
			grafik_xy(nx,ny,grafik_inv);
					
			nx += inc_x;
			tmp1 = tmp1 + tmp2;
			if(tmp1 > 0)
			{
				ny += inc_y;
				tmp1 = tmp1 + schwelle;
			}
		}
	}
	else
	{
		tmp2 = -tmp2;
		tmp1 = tmp1 + tmp1;
		schwelle = tmp2 + tmp2;
		while(ny != y2)
		{
			grafik_xy(nx,ny,grafik_inv);
			
			ny += inc_y;
			tmp2 = tmp2 + tmp1;
			if(tmp2 > 0)
			{
				nx += inc_x;
				tmp2 = tmp2 + schwelle;
			}
		}
	}

	grafik_xy(nx,ny,grafik_inv);
}

//Artur Pundsack
void grafik_circle(unsigned char x1,unsigned char y1,unsigned char r)
{
    char x, y, d, dx, dxy;

    x = 0;
    y = r;
    d = 1 - r;
    dx = 3;
    dxy = -(r << 1) + 5;

    while(y >= x)
    {
    		
    		grafik_xy(x1+x, y1+y,grafik_inv);
		grafik_xy(x1+y, y1+x,grafik_inv);
		grafik_xy(x1+y, y1-x,grafik_inv);
		grafik_xy(x1+x, y1-y,grafik_inv);
	        grafik_xy(x1-x, y1-y,grafik_inv);
		grafik_xy(x1-y, y1-x,grafik_inv);
		grafik_xy(x1-y, y1+x,grafik_inv);
		grafik_xy(x1-x, y1+y,grafik_inv);

		if(d<0)
		{
			d = d + dx;
			dx = dx + 2;
			dxy = dxy + 2;
			x++;
		}
		else
		{
			d = d + dxy;
			dx = dx + 2;
			dxy = dxy + 4;
			x++;
			y--;
		}
	}
}

void grafik_wr_cmd(unsigned char cmd, unsigned char chip)
{
	clvar = (clvar | 0x02) & 0xFE;					//Instruktionen fr Grafikcontroller
	CLWR = clvar;
		
	if(chip == 0) graWR1 = cmd;
	else if(chip != 0) graWR2 = cmd;
	grafik_wbusy();
}

void grafik_wr_data(unsigned char val, unsigned char chip)
{
	clvar = clvar | 0x03;						//Daten fr Grafikcontroller
	CLWR = clvar;
		
	if(chip == 0) graWR1 = val;
	else if(chip != 0) graWR2 = val;
	grafik_wbusy();
}

unsigned char grafik_rd_cmd(unsigned char chip)
{
	clvar = (clvar | 0x02) & 0xFE;					//Instruktionen fr Grafikcontroller
	CLWR = clvar;
		
	if(chip == 0) return graRD1;
	else if(chip != 0) return graRD2;
	grafik_wbusy();
}

unsigned char grafik_rd_data(unsigned char chip)
{
	clvar = clvar | 0x03;						//Daten fr Grafikcontroller
	CLWR = clvar;
		
	if(chip == 0) return graRD1;
	else if(chip != 0) return graRD2;
	grafik_wbusy();
}

#endif
